var App = function(publicUrl, opts) {
  opts = opts || {};
  this.publicUrl = publicUrl;
  this.rootUrl = null;
  this.apiUrl = null;
  this.userInfo = {};
  this.userShares = [];
  this.userWorkspaces = [];
  this.action = "new";
  
  this.onPremOnly = !!opts.onPremOnly;
  this.mode = opts.mode;
  PersistentStorage.load();
  this.customServerUrl = null;
  this.setCustomServerUrl(localStorage.getItem("customServerUrl"));
}

App.prototype.setCustomServerUrl = function(customServerUrl) {
  this.customServerUrl = customServerUrl;
  if (customServerUrl) {
    this.apiUrl = customServerUrl + "/api";
    this.rootUrl = customServerUrl;
    localStorage.setItem("customServerUrl", customServerUrl);
  } else if (!this.onPremOnly) {
    this.apiUrl = this.publicUrl + "/api";
    this.rootUrl = this.publicUrl;
    localStorage.removeItem("customServerUrl");
  }
}

App.prototype.init = function(rootPage) {
  var that = this;

  that.mainContainer = document.getElementsByClassName("main-container")[0];
  that.mainLoader = document.getElementById("mainLoader");
  that.setLoading(true);

  document.getElementById("closeBtn").addEventListener("click", (e) => {
    that.close();
  }, false);

  document.addEventListener('keypress', (e) => {
    if (e.keyCode == 27) { //ESC
      that.close();
    }
  }, false);

  that.initServices();
  that.loadRootPage(rootPage);
}

App.prototype.setLoading = function(loading) {
  this.mainLoader.style.display = loading ? "block" : "none";
}

App.prototype.discoverIndigoServer = function() {
  if (window.exported) {
    var srvRecord = '_indigo-server._tcp';
    var indigoServer = window.exported.resolveServer(srvRecord);
    if (indigoServer) {
      try {
        const url = new URL("http://" + indigoServer);
        if (url.port.toString().endsWith("443")) {
          url.protocol = "https";
        }

        // Get url string and remove trailing slash
        const urlString = url.toString().replace(/\/$/, "");
        this.setCustomServerUrl(urlString);
        return urlString;
      } catch (err) {
        console.warn("Error while parsing discovered URL: " + indigoServer);
      }
    }
  }
}

App.prototype.exportAssets = function() {
  if (!window.exported) {
    console.error("No exported functions");
    return null;
  }

  return JSON.parse(window.exported.exportAssets());
}

App.prototype.initServices = function() {
  var that = this;

  that.resourceLoader = new ResourceLoader();
  that.navigator = new Navigator();
  that.localization = new Localization();
}

App.prototype.loadRootPage = function(rootPage) {
  var that = this;
  that.setLoading(true);

  that.hasValidAccessToken().
    then((result) => {
      switch (rootPage) {
        case "login":
          loadMainPage(result);
          break;
        case "sync":
          this.mode = "sync";
          loadSyncPage(result);
          break;
        default:
          break;
      }

      document.getElementsByClassName("header-container")[0].style.display = "grid";
    })
    .catch((err) => {
      that.setLoading(false);
      var errorDialog = document.createElement("div");
      errorDialog.className = "error-dialog";
      errorDialog.style.display = "block";
      errorDialog.innerHTML = that.localization.strings.Error + ": " + err;

      that.mainContainer.appendChild(errorDialog);
    });

  function loadMainPage(hasValidAccessToken) {
    that.resourceLoader.loadSet([
      "views/login.html",
      "views/publishEditor.html",
      "views/publishing.html",
      "views/completed.html",
      "views/failed.html",
      "views/shareContainer.html",
      "sketch/sketchData.js",
      "views/createApp.html",
      "views/creating.html",
      "views/onpremiseLogin.html"
    ]).then(function(templates) {
        that.pages = {
            login: {
              template: templates[0],
              model: new Login(that)
            },
            publishEditor: {
              template: templates[1],
              model: new PublishEditor(that)
            },
            publishing: {
              template: templates[2],
              model: new Publishing(that)
            },
            completed: {
              template: templates[3],
              model: new Completed(that)
            },
            failed: {
              template: templates[4],
              model: new Failed(that)
            },
            shareContainer: {
              template: templates[5]
            },
            createApp: {
              template: templates[7],
              model: new Create(that)
            },
            creating: {
              template: templates[8],
              model: new Creating(that)
            },
            onpremiseLogin: {
              template: templates[9],
              model: new OnPremiseLogin(that)
            }
        };

        eval(templates[6]);

        that.loadData().then(() => {
          if (!hasValidAccessToken) {
            that.navigator.navigate(that.mainContainer, that.pages.login);
            return;
          }

          if (!that.isUserEntitled()) {
            that.pages.failed.model.error = that.localization.strings.TrialExpired;
            that.navigator.navigate(that.mainContainer, that.pages.failed);
            return;
          }

          that.afterLogin();
        }, (err) => {
          that.setLoading(false);
          that.pages.failed.model.error = err;
          that.navigator.navigate(that.mainContainer, that.pages.failed);
        });
    }).catch(function (err) {
      that.setLoading(false);
      console.error("Error loading templates:", err);
    });
  }

  function loadSyncPage(hasValidAccessToken) {
    that.resourceLoader.loadSet([
      "views/synchronize.html",
      "views/failed.html",
      "views/login.html",
      "views/publishEditor.html",
      "views/publishing.html",
      "views/completed.html",
      "views/shareContainer.html",
      "sketch/sketchData.js",
    ]).then(function(templates) {
          that.pages = {
              sync: {
                template: templates[0],
                model: new Sync(that)
              },
              failed: {
                template: templates[1],
                model: new Failed(that)
              },
              login: {
                template: templates[2],
                model: new Login(that)
              },
              publishEditor: {
                template: templates[3],
                model: new PublishEditor(that)
              },
              publishing: {
                template: templates[4],
                model: new Publishing(that)
              },
              completed: {
                template: templates[5],
                model: new Completed(that)
              },
              shareContainer: {
                template: templates[6]
              }
          };

        eval(templates[7]);

        that.loadData().then(() => {
          if (!hasValidAccessToken) {
            that.pages.failed.model.error = that.localization.strings.NeedLogin;
            that.navigator.navigate(that.mainContainer, that.pages.failed);
            return;
          }

          if (!that.isUserEntitled()) {
            that.pages.failed.model.error = that.localization.strings.TrialExpired;
            that.navigator.navigate(that.mainContainer, that.pages.failed);
            return;
          }

          that.afterLogin();
        }, (err) => {
          that.pages.failed.model.error = err;
          that.navigator.navigate(that.mainContainer, that.pages.failed);
        });
    }).catch(function (err) {
      that.setLoading(false);
      console.error("Error loading templates:", err);
    });
  }

}

App.prototype.afterLogin = function(userInfo) {
  this.setLoading(true);
  this.userInfo = JSON.parse(localStorage.getItem("userInfo")).data;
  this.showTrialDays(this.userInfo);

  let needsToPublish = this.mode === "publish";
  if (this.mode === "sync") {
    // Sync, check if current file is the same as lastPublished
    const lastSketchFileRaw = localStorage.getItem("lastSketchFile");
    const lastSketchFile = lastSketchFileRaw ? JSON.parse(lastSketchFileRaw) : null;
    const sameFile = lastSketchFile &&
                     sketchData.sketchFile.id == lastSketchFile.id &&
                     sketchData.sketchFile.filePath === lastSketchFile.filePath;
    if (!sameFile) {
        console.info("Current file differs from last publish, go to publish", sketchData.sketchFile, lastSketchFile);
        needsToPublish = true;
    }
  }

	if (this.mode === "create") {
		this.prepareCreatePage()
		.then(() => {
			this.setLoading(false);
			  this.navigator.navigate(this.mainContainer, this.pages.createApp);
		}).catch((err) => {
			this.setLoading(false);
			this.pages.failed.model.error = err;
			this.navigator.navigate(this.mainContainer, this.pages.failed);
		});
	} else if (needsToPublish) {
    this.preparePublishPage()
      .then(() => {
        this.setLoading(false);
        this.navigator.navigate(this.mainContainer, this.pages.publishEditor);
      }).catch((err) => {
        this.setLoading(false);
        this.pages.failed.model.error = err;
        this.navigator.navigate(this.mainContainer, this.pages.failed);
      });
  } else {
    this.prepareSyncPage()
      .then(() => {
        this.setLoading(false);
        this.navigator.navigate(this.mainContainer, this.pages.sync);
      }).catch((err) => {
        this.setLoading(false);
        this.pages.failed.model.error = err;
        this.navigator.navigate(this.mainContainer, this.pages.failed);
      });
  }
}

App.prototype.preparePublishPage = function() {
  return (
    getAllShares(this.apiUrl, this.userInfo.profile.username, this.userInfo.token)
    .then((r) => {
      this.userShares = r.shares.filter((s) => { return s.editor == 1; });
      localStorage.setItem("userShares", JSON.stringify(this.userShares));

      this.userWorkspaces = r.workspaces;
      localStorage.setItem("userWorkspaces", JSON.stringify(this.userWorkspaces));

      return getDeviceIds(this.apiUrl + "/ibp/graphql")
        .then((devices) => {
          localStorage.setItem("devices", devices);
        });
    })
  );
}

App.prototype.prepareSyncPage = function() {
  return (
    getUserWorkspaces(this.apiUrl, this.userInfo.profile.username, this.userInfo.token)
    .then((workspaces) => {
      this.userWorkspaces = workspaces;
      localStorage.setItem("userWorkspaces", JSON.stringify(this.userWorkspaces));
    })
  );
}

App.prototype.prepareCreatePage = function() {
  return (
    getAllShares(this.apiUrl, this.userInfo.profile.username, this.userInfo.token)
    .then((r) => {
      this.userShares = r.shares.filter((s) => { return s.editor == 1; });
      localStorage.setItem("userShares", JSON.stringify(this.userShares));

      this.userWorkspaces = r.workspaces;
      localStorage.setItem("userWorkspaces", JSON.stringify(this.userWorkspaces));

      
      var workspaceId = null
      if (this.userWorkspaces.length > 0){
        workspaceId = this.userWorkspaces[0].workspaceId;
      }


      return getPortsIds(`${window.appbuilderWorkspaceUrl.Production}${workspaceId}/viewports`, this.userInfo.token)
        .then((ports) => {
          localStorage.setItem("ports", ports);
        });
    })
  );
}

App.prototype.isUserEntitled = function(userInfo) {
  var storedInfo = localStorage.getItem("userInfo");
  if (!storedInfo) {
    return false;
  }

  var userInfo = JSON.parse(storedInfo).data;
  if (userInfo.profile.license.subscription == 1) { //User is in Trial
    var daysLeft = this.pages.login.model.calculateTrialDays(userInfo);
    if (daysLeft <= 0) {

      return false;
    }
  }

  if (userInfo.profile.license.tier == 1) { //User has changed to Essentials license
    return false;
  }

  return true;
}

App.prototype.hasValidAccessToken = function(token) {
  var that = this;
  if (!token) {
    return that.getToken()
        .then((tokenResponse) => {
          var url = `${window.loginVerifyUrl.Production}`;
          return httpGet(url, tokenResponse)
            .then((result) => {
              const obj = JSON.parse(result);
              if (obj.status == 0) {
                localStorage.setItem("userInfo", result);
                return true;
              } else {
                localStorage.removeItem("userInfo");
                return false;
              }
            });
        }).catch((err) => {
          return false;
      });
  } else {
    // if token available, just verify it agaisnt login/verify
    var url = `${window.loginVerifyUrl.Production}`;
    return httpGet(url, token)
      .then((result) => {
        const obj = JSON.parse(result);
        if (obj.status == 0) {
          localStorage.setItem("userInfo", result);
          return true;
        } else {
          localStorage.removeItem("userInfo");
          return false;
        }
      });
  }
}

App.prototype.getToken = function() {
  var that = this;
  return new Promise((resolve, reject) => {
    try {
      const tokenBuffer = localStorage.getItem("iamToken");
      const tokenInfo = tokenBuffer ? JSON.parse(tokenBuffer) : null;
      const expiresAt = tokenInfo ? new Date(tokenInfo["expires_at"]) : null
      if (expiresAt && (expiresAt - new Date()) / 1000 > 10 ) {
        // access token is still valid
        resolve(tokenInfo.access_token);
      } else {
        // Use refresh token to attempt to get a new access token
        if (tokenInfo) {
          var refreshToken = tokenInfo["refresh_token"];
          var loginUrl = `${window.loginUrl.Production}`;
          xhrPolling(`${loginUrl}/credentials-refresh`, 'POST', `refreshToken=${refreshToken}`, 3000).then((tokenResponse) => {
            that.saveTokenResponse(tokenResponse, refreshToken);
            resolve(tokenResponse.accessToken);
          })
          .catch(err => {
            reject(err);
          })
        } else {
          reject("No token found in storage")
        }
      }
      } catch (e) {
        reject(e);
        // no token stored or failed to renew with refresh token
      }
  });
}

App.prototype.saveTokenResponse = function(tokenResponse, refreshToken) {
	var time = new Date();
	time.setSeconds(time.getSeconds() + parseInt(tokenResponse.expiresIn));
	var tokenInfo = {
		access_token: tokenResponse.accessToken,
		expires_at: time,
		refresh_token: tokenResponse.refreshToken || refreshToken
	};
	localStorage.setItem("iamToken", JSON.stringify(tokenInfo));
	localStorage.setItem("id_token", tokenResponse.identityToken);
}

App.prototype.showTrialDays = function(userInfo) {
  if (userInfo.profile.license.subscription == 1) { //Show trial days
    const trialContainer = document.getElementsByClassName("trial-info-container")[0],
        trialDaysLabel = document.getElementById("trialDaysLabel");

    trialContainer.style.visibility = "visible";
    trialDaysLabel.innerHTML = this.localization.strings.TrialDaysLabel;

    var trialDays = document.getElementById("trialDays")
    trialDays.innerHTML = this.pages.login.model.calculateTrialDays(userInfo);
  }
}

App.prototype.loadData = function() {
  var promises = [loadSketchFile(), loadThumbs()];
  return Promise.all(promises)
    .then((resultArr) => {
      sketchData.sketchFile.file = resultArr[0];
      sketchData.screens.forEach((s, i) => {
        s.image = resultArr[1][i];
      });
    });
}

App.prototype.close = function() {
  PersistentStorage.save();
  // The webview owner intercepts the navigation and closes the dialog
  window.location.href = "https://localhost/close";
}
