var PersistentStoragePersistKeys = ["customServerUrl", "accessToken", "iamToken", "userInfo", "publishParams", "lastSketchFile"];

/** 
 * static methods to persist keys from localStorage and
 * load them into localStorage.
 * It makes use of exported functions in the backend code (Mocha bridge)
 */
var PersistentStorage = class {    
    static load() {
        if (window.exported) {
            const storageString = window.exported.loadStorage();
            if (!storageString) {
                console.info("No persisted storage");
                return;
            }

            const data = JSON.parse(storageString);
            for (const key of Object.keys(data)) {
                localStorage.setItem(key, data[key]);
            }
        } else {
            console.warn("No exported services");
        }
    }

    static save() {
        if (window.exported) {
            const data = {};
            for (let i = localStorage.length; i--; i >= 0) {
                const key = localStorage.key(i);
                if (PersistentStoragePersistKeys.includes(key)) {
                    data[key] = localStorage.getItem(key);
                }
            }

            const storageString = JSON.stringify(data);
            window.exported.saveStorage(storageString);
        } else {
            console.warn("No exported services");
        }
    }
}
