var Create = function(app) {
  var that = this;
  that.app = app;
}

Create.prototype.init = function() {
  var that = this,
    loadingIndicator = document.querySelector(".Loading.horizontal"),
    createBtn = document.querySelector("#createBtn");

  createBtn.addEventListener("click", (e) => {
    this.clearErrorDialog();
    that.initCreate.call(this);
  }, false);

  that.initDefaults();
  that.loadWorkspaces();
  that.loadPorts();
}

Create.prototype.initCreate = function() {
  var that = this,
    loadingIndicator = document.querySelector(".Loading.horizontal");
  file = sketchData.sketchFile;

  loadingIndicator.style.display = "block";
  var url = `${window.createFromSketchUrl.Production}`;

  var token = JSON.parse(localStorage.getItem("iamToken")).access_token;

  try {

    var params = that.populateParams();
    that.loadCreatingPage();
      create(url, file, params, token)
        .then((r) => {
          loadingIndicator.style.display = "none"; 
          var result = JSON.parse(r);
          params.project.id = result.shareId;
          that.app.publishedShareId = result.shareId;
          that.app.publishedShareTitle = params.project.name;
          delete params.assets;
        var appUrl = `${window.appUrl.Production}`;
        appUrl = `${appUrl}${result.application.id}`;

        var workspaceUrl;
        if (result.application.workspaceId !== "null") {
          workspaceUrl = `${window.workspaceUrl.Production}`;
          workspaceUrl = `${workspaceUrl}${result.application.workspaceId}/apps`;
        } else {
            workspaceUrl = `${window.appsHomeUrl.Production}`;
        }
        
          
          var result =
        {
          shareId: appUrl,
          shareLink: appUrl,
          workspaceUrl: workspaceUrl
        }

          that.loadCompletedPage(result);
        })
        .catch((err) => {
          that.showErrorDialog(err);
        });
  } catch (err) {
    that.showErrorDialog(err);
    return;
  }
}

Create.prototype.initDefaults = function() {
  var that = this,
    createBtn = document.getElementById("createBtn"),
    appTitleInput = document.getElementById("appTitleInput"),
    fileParts = sketchData.sketchFile.filePath.split("/");

  var fileName = fileParts[fileParts.length - 1].split(".")[0];
  appTitleInput.value = decodeURI(fileName);

  appTitleInput.addEventListener("keyup", (e) => {
    var value = e.target.value;
    if (value.length > 0 && value.length <= 256) {
      createBtn.removeAttribute("disabled");
    } else {
      createBtn.setAttribute("disabled", "disabled");
    }
  }, false);
}

Create.prototype.loadWorkspaces = function() {
  var that = this,
    combo = document.getElementById("workspaceCombo"),
    createOption = function(combo, workspace) {
      var option = document.createElement("div");
      option.innerHTML = workspace.title;
      option.className = "selectable-dropdown-items";
      option.setAttribute("data-id", workspace.workspaceId);

      var itemsContainer = combo.querySelector(".select-items");
      itemsContainer.appendChild(option);
    };

  var userWorkspaces = JSON.parse(localStorage.getItem("userWorkspaces"));
  userWorkspaces
    .filter((w) => {
      return w.archivedAt == null;
    })
    .sort((a, b) => {
      var first = a.title.toUpperCase();
      var second = b.title.toUpperCase();
      if (first < second) {
        return -1;
      }

      if (second < first) {
        return 1;
      }

      return 0;
    })
    .forEach((w) => {
      createOption(combo, w);
    });

  initializeCustomSelect("workspaceCombo");
}

Create.prototype.loadPorts = function () {
  var portsData = JSON.parse(localStorage.getItem("ports"));
  var that = this,
    combo = document.getElementById("portsCombo"),
    createOption = function (combo, port) {
      var option = document.createElement("div");
      option.innerHTML = port.name;
      option.className = "selectable-dropdown-items";
      option.setAttribute("data-id", port.id);

      var itemsContainer = combo.querySelector(".select-items");
      itemsContainer.appendChild(option);
    };

    portsData.forEach((p) => {
    createOption(combo, p);
  });

  initializeCustomSelect("portsCombo");
  setSelectedValue("portsCombo", portsData[2].id);
}

Create.prototype.loadCreatingPage = function() {
  var that = this;
  that.app.navigator.navigate(that.app.mainContainer, that.app.pages.creating);
}

Create.prototype.loadCompletedPage = function(result) {
  var that = this;
  that.app.shareId = result.shareId;
  that.app.shareLink = result.shareLink;
  that.app.workspaceUrl = result.workspaceUrl;

  that.app.navigator.navigate(that.app.mainContainer, that.app.pages.completed);
}

Create.prototype.showErrorDialog = function(error) {
  var that = this,
    errorSpan = document.getElementById("errorSpan");

  document.querySelector(".Loading.horizontal").style.display = "none";
  if (error == "Unauthorized") {
    error = that.app.localization.strings.SignInFailed;
  }

  errorSpan.innerHTML = error;
  errorSpan.style.display = "block";
  errorSpan.classList.remove("info-dialog");
  errorSpan.classList.add("error-dialog");
}

Create.prototype.clearErrorDialog = function(error) {
  var that = this,
    errorSpan = document.getElementById("errorSpan");

  errorSpan.innerHTML = '';
  errorSpan.style.display = 'none';
}

Create.prototype.devicesDropdownValueChanged = function (result) {
  var deviceDimensions = document.getElementById("deviceDimensions");
  var dimenstionsContainer = document.getElementsByClassName("create-dimensions-container")[0];

  if (result == null || result.value == null || result.value.indexOf("generic") < 0) {
    deviceDimensions.style.display = "none";
    dimenstionsContainer.style.display = "block";
  } else {
    deviceDimensions.style.display = "grid";
    dimenstionsContainer.style.display = "grid";
  }
}


Create.prototype.populateParams = function() {
  var that = this;

  var params = {
    "project": {
      "name": null,
      "viewPort": getSelectedPortSize("portsCombo"),
      "homeId": getHomeScreenId()
    },
    "sketchMetadata": {
      "pages": [sketchData.pages]
    },
    "create": true
  };

  function getHomeScreenId() {
    var home = sketchData.screens.filter((s) => s.isHome)[0];
    if (home != null) {
      return home.id;
    }

    return sketchData.screens.length > 0 ? sketchData.screens[0].id : null;
  }

  params = that.populateNewShareParams(params);

  return params;
}

Create.prototype.populateNewShareParams = function(params) {
  var that = this;
  var appTitleInput = document.getElementById("appTitleInput");
  params.project.name = appTitleInput.value;

  var workspaceCombo = document.querySelector("#workspaceCombo .select-selected");
  params.workspaceId = workspaceCombo.innerHTML.toLowerCase() == "personal" ? null : workspaceCombo.getAttribute("data-id");
  
  return params;
}