var Login = function(app) {
  var that = this;
  that.app = app;
  that.app.setCustomServerUrl(null);
}

Login.prototype.init = function() {
  var that = this,
      tryAgain = document.getElementById("tryAgain"),
      haveOwnServer = document.getElementById("haveOwnServer"),
      newAccountLink = document.getElementById("newAccountLink"),
      loadingIndicator = document.querySelector(".Loading.horizontal");

  loadingIndicator.style.display = "none";
  tryAgain.addEventListener("click", (e) => that.handleSignIn.call(that), false);
  haveOwnServer.addEventListener("click", (e) => { that.handleOwnServer(e) }, false);
  newAccountLink.addEventListener("click", (e) => {
    window.location.href = "https://localhost/link=" + encodeURI(that.app.publicUrl + "/register");
    e.preventDefault();
  }, false);

  if (this.app.onPremOnly) {
    document.querySelector(".login-form").classList.add("login-premises-only");
  }
  this.setPremises(false);
  document.addEventListener('keypress', (e) => {
    if (e.keyCode === 13) { //Enter
      tryAgain.click();
    }
  }, false);
  that.handleSignIn();
}

Login.prototype.handleLink = function(e) {
  window.location.href = "https://localhost/link=" + encodeURI(e.target.href);
  e.preventDefault();
}

Login.prototype.handleOwnServer = function(e) {
  var that = this;
  that.setPremises(true);
  that.app.navigator.navigate(that.app.mainContainer, that.app.pages.onpremiseLogin);
}

Login.prototype.setPremises = function(isPremises) {
  this.isPremises = isPremises;
  const loginForm = document.querySelector(".login-form");
  loginForm.classList.remove("login-premises");
  loginForm.classList.add("login-public");
}

Login.prototype.signIn = function() {
  return new Promise((resolve, reject) => {
      var that = this;
      const url = `${window.loginUrl.Production}`;

      try {
        xhrPolling(`${url}/request-id`, 'GET')
        .then((res) => {
           const loginUrl = `${url}/login?requestId=${res}`;
           document.location.href = "https://localhost/link=" + loginUrl;
           return xhrPolling(`${url}/credentials`, 'POST', `requestId=${res}`)
         })
         .then((tokenResponse) => {
           return that.app.hasValidAccessToken(tokenResponse.accessToken).then((isValid) => {
           if (isValid) {
             that.app.saveTokenResponse(tokenResponse, tokenResponse.refreshToken);
             resolve(tokenResponse.accessToken);
           } else {
             reject("Token is not valid.");
           }
           })
         }).catch(err => {
          reject(err);
        })
      } catch(e) {
        reject(e);
      } finally {
        // dialog.close();
      }
  });
}

Login.prototype.handleSignIn = function() {

  var that = this,
      errorSpan = document.getElementById("errorSpan"),
      signInBtn = document.getElementById("signInBtn"),
      loadingIndicator = document.querySelector(".Loading.horizontal");

  errorSpan.innerHTML = "";
  errorSpan.style.display = "none";
  tryAgain.disabled = true;
  loadingIndicator.style.display = "block";

  this.signIn().then((res) => {
    const userInfoResult = JSON.parse(localStorage.getItem("userInfo")).data;
    if (that.isEssentialsLicense(userInfoResult) || that.isTrialExpired(userInfoResult)) {
      // ^ above conditions already set error messages, we don't proceed
      loadingIndicator.style.display = "none";
      return;
    }
    that.app.afterLogin();
      })
  .catch(err => {
    loadingIndicator.style.display = "none";
    that.showErrorDialog(err.message || err);
  })
};

Login.prototype.isEssentialsLicense = function(userInfo) {
  var that = this,
      errorSpan = document.getElementById("errorSpan"),
      signInBtn = document.getElementById("signInBtn");
  if (userInfo.profile.license.tier == 1) { // User has an Essentials license

    errorSpan.innerHTML = "";

    var span = document.createElement("span");
    span.innerHTML = that.app.localization.strings.UpgradeText;

    var link = document.createElement("a");
    link.href = "https://www.infragistics.com/products/indigo-design/pricing";
    link.innerHTML = that.app.localization.strings.UpgradeTextHref;
    span.appendChild(link);

    errorSpan.appendChild(span);

    link.addEventListener("click", (e) => { that.handleLink(e) }, false);

    errorSpan.style.display = "block";
    errorSpan.classList.add("info-dialog");
    errorSpan.classList.remove("error-dialog");
    signInBtn.disabled = false;
    return true;
  }

  return false;
}

Login.prototype.isTrialExpired = function(userInfo) {
  var that = this;

  if (userInfo.profile.license.subscription == 1) { //User is in Trial subscription
    if (that.calculateTrialDays(userInfo) <= 0) {
      that.showErrorDialog(that.app.localization.strings.TrialExpired);
      return true;
    }
  }

  return false;
}

Login.prototype.showErrorDialog = function(error) {
  var that = this,
      errorSpan = document.getElementById("errorSpan"),
      signInBtn = document.getElementById("signInBtn");

  if (error == "Unauthorized" || error == "Request error. Status code: 0") {
    error = that.app.localization.strings.SignInFailed;
  }

  errorSpan.innerHTML = error;
  errorSpan.style.display = "block";
  errorSpan.classList.remove("info-dialog");
  errorSpan.classList.add("error-dialog");
}

Login.prototype.showWaitUI = function(msg) {
  var that = this,
			loginState = document.querySelector(".waitState");

	loginState.style.display = "block";
  loginState.innerHTML = msg;
}

Login.prototype.calculateTrialDays = function(userInfo) {
  var that = this,
      expireDate = new Date(userInfo.profile.license.expiresAt);

  return Math.round((expireDate - new Date()) / (1000 * 60 * 60 * 24));
}
