var OnPremiseLogin = function(app) {
    var that = this;
    that.app = app;
  }
  
  OnPremiseLogin.prototype.init = function() {
    this.showLoader();
    var that = this,
        serverUrlInput = document.getElementById("serverURL"),
        signInBtn = document.getElementById("signInBtn"),
        forgotLink = document.getElementById("forgotLink"),
        haveOwnServerLink = document.getElementById("haveOwnServer"),
        dontHaveOwnServerLink = document.getElementById("dontHaveOwnServer"),
        newAccountLink = document.getElementById("newAccountLink"),
        loadingIndicator = document.querySelector(".Loading.horizontal");

  
    signInBtn.addEventListener("click", (e) => that.handleSignIn.call(that), false);
    forgotLink.addEventListener("click", (e) => { that.handleLink(e) }, false);
    haveOwnServerLink.addEventListener("click", (e) => { that.handleOwnServer(e) }, false);
    dontHaveOwnServerLink.addEventListener("click", (e) => { that.handleDontOwnServer(e) }, false);
    newAccountLink.addEventListener("click", (e) => {
      window.location.href = "https://localhost/link=" + encodeURI(that.app.publicUrl + "/register");
      e.preventDefault();
    }, false);
  
    serverUrlInput.addEventListener("blur", () => {
      if (!serverUrlInput.value) {
        // re-trigger auto discover
        this.setPremises(true);
      }
    });
  
    if (this.app.onPremOnly) {
      document.querySelector(".login-form").classList.add("login-premises-only");
    }
  
    serverUrlInput.value = that.app.customServerUrl || "";
    this.setPremises(!!that.app.customServerUrl || that.app.onPremOnly);
  
    document.addEventListener('keypress', (e) => {
      if (e.keyCode === 13) { //Enter
        signInBtn.click();
      }
    }, false);
    this.setPremises(true);
    this.hasValidAccessToken().
      then((result) => {
        that.app.afterLogin();
      })
      .catch((err) => {
        this.hideLoader();
      });
  }
  
  OnPremiseLogin.prototype.handleLink = function(e) {
    window.location.href = "https://localhost/link=" + encodeURI(e.target.href);
    e.preventDefault();
  }
  
  OnPremiseLogin.prototype.handleOwnServer = function(e) {
    this.setPremises(true);
  }
  
  OnPremiseLogin.prototype.handleDontOwnServer = function(e) {
    var that = this;
    this.setPremises(false);
    that.app.navigator.navigate(that.app.mainContainer, that.app.pages.login);
  }
  
  OnPremiseLogin.prototype.setPremises = function(isPremises) {
    this.isPremises = isPremises;
    const loginForm = document.querySelector(".login-form");
    if (isPremises) {
      loginForm.classList.remove("login-public");
      loginForm.classList.add("login-premises");
    } else {
      loginForm.classList.remove("login-premises");
      loginForm.classList.add("login-public");
    }
  
    var serverUrlInput = document.getElementById("serverURL");
    if (isPremises && !serverUrlInput.value) {
      this.app.discoverIndigoServer();
      if (this.app.customServerUrl) {
        serverUrlInput.value = this.app.customServerUrl;
      }
    }
  
    if (isPremises && serverUrlInput.value) {
      document.getElementById("emailField").focus();
    }
  }
  
  OnPremiseLogin.prototype.validateServerUrl = function() {
    if (!this.isPremises) {
      return Promise.resolve(this.app.publicUrl);
    }
  
    const inputUrl = document.getElementById("serverURL").value;
    if (!inputUrl) {
      return Promise.reject("validate-no-url");
    }
  
    let versionUrl = null;
    try {
      versionUrl = new URL("/api/version", inputUrl);
    } catch (err) {
      return Promise.reject("validate-invalid-url");
    }
  
    return jsonRequest(versionUrl)
      .catch(err => {
        throw "validate-not-indigo-server";
      })
      .then((res) => {
        if (res.status === 0) {
          return versionUrl.origin;
        } else {
          throw "validate-not-indigo-server";
        }
      });
  }

  OnPremiseLogin.prototype.hasValidAccessToken = function() {
    return new Promise((resolve, reject) => {
      var that = this;
      const token = localStorage.getItem("accessToken");
      if (token !== null) {
        httpGet(that.app.apiUrl + '/account/login/verify', token)
        .then((result) => {
          const userInfoResult = JSON.parse(result);
          if (that.isEssentialsLicense(userInfoResult.data) || that.isTrialExpired(userInfoResult.data)) {
            // ^ above conditions already set error messages, we don't proceed
            loadingIndicator.style.display = "none";
            return;
          }
  
          localStorage.setItem("userInfo", JSON.stringify(userInfoResult));
          localStorage.setItem("accessToken", userInfoResult.data.token);
          resolve(true);
        })
        .catch((err) => {
          reject(err);
        });
      } else {
        reject();
      }
    });
  }
  
  OnPremiseLogin.prototype.handleSignIn = function() {
  
    var that = this,
        username = document.getElementById("emailField").value,
        password = document.getElementById("passwordField").value,
        errorSpan = document.getElementById("errorSpan"),
        signInBtn = document.getElementById("signInBtn"),
        loadingIndicator = document.querySelector(".Loading.horizontal");
  
    errorSpan.innerHTML = "";
    errorSpan.style.display = "none";
    signInBtn.disabled = true;
    loadingIndicator.style.display = "block";
  
    let step = "validate-url";
    this.validateServerUrl()
      .then((serverUrl) => {
          that.app.setCustomServerUrl(that.isPremises ? serverUrl : null);
          step = "sign-in";
          return jsonRequest(that.app.apiUrl + '/account/login', { username, password })
      }).then((res) => {
          httpGet(that.app.apiUrl + '/account/login/verify', res.data.token)
          .then((result) => {
            const userInfoResult = JSON.parse(result);
            if (that.isEssentialsLicense(userInfoResult.data) || that.isTrialExpired(userInfoResult.data)) {
              // ^ above conditions already set error messages, we don't proceed
              loadingIndicator.style.display = "none";
              return;
            }

            localStorage.setItem("userInfo", JSON.stringify(userInfoResult));
            localStorage.setItem("accessToken", userInfoResult.data.token);
            that.app.afterLogin();
          });
      }).catch((err) => {
        loadingIndicator.style.display = "none";
  
          let errMessage = err.message || err;
          if (err == "validate-no-url") {
            errMessage = that.app.localization.strings.ErrorUrlRequired;
          } else if (err == "validate-invalid-url") {
            errMessage = that.app.localization.strings.ErrorUrlValid;
          } else if (err == "validate-not-indigo-server") {
            errMessage = that.app.localization.strings.ErrorNotIndigoServer;
          } else if (step === "sign-in" && err instanceof RequestError && err.status === 401) {
            errMessage = that.app.localization.strings.SignInFailed;
          }
  
          that.showErrorDialog(errMessage);
      });
  }

  
  OnPremiseLogin.prototype.isEssentialsLicense = function(userInfo) {
    var that = this,
        errorSpan = document.getElementById("errorSpan"),
        signInBtn = document.getElementById("signInBtn");
  
    if (userInfo.profile.license.tier == 1) { //User has an Essentials license
  
      errorSpan.innerHTML = "";
  
      var span = document.createElement("span");
      span.innerHTML = that.app.localization.strings.UpgradeText;
  
      var link = document.createElement("a");
      link.href = "https://www.infragistics.com/products/indigo-design/pricing";
      link.innerHTML = that.app.localization.strings.UpgradeTextHref;
      span.appendChild(link);
  
      errorSpan.appendChild(span);
  
      link.addEventListener("click", (e) => { that.handleLink(e) }, false);
  
      errorSpan.style.display = "block";
      errorSpan.classList.add("info-dialog");
      errorSpan.classList.remove("error-dialog");
      signInBtn.disabled = false;
      return true;
    }
  
    return false;
  }
  
  OnPremiseLogin.prototype.isTrialExpired = function(userInfo) {
    var that = this;
  
    if (userInfo.profile.license.subscription == 1) { //User is in Trial subscription
      if (that.calculateTrialDays(userInfo) <= 0) {
        that.showErrorDialog(that.app.localization.strings.TrialExpired);
        return true;
      }
    }
  
    return false;
  }
  
  OnPremiseLogin.prototype.showErrorDialog = function(error) {
    var that = this,
        errorSpan = document.getElementById("errorSpan"),
        signInBtn = document.getElementById("signInBtn");
  
    if (error == "Unauthorized") {
      error = that.app.localization.strings.SignInFailed;
    }
  
    errorSpan.innerHTML = error;
    errorSpan.style.display = "block";
    errorSpan.classList.remove("info-dialog");
    errorSpan.classList.add("error-dialog");
  
    signInBtn.disabled = false;
  }
  
  OnPremiseLogin.prototype.calculateTrialDays = function(userInfo) {
    var that = this,
        expireDate = new Date(userInfo.profile.license.expiresAt);
  
    return Math.round((expireDate - new Date()) / (1000 * 60 * 60 * 24));
  }

  OnPremiseLogin.prototype.showLoader = function(e) {
    var loginLoader = document.querySelector(".login-loader"),
      loginForm = document.querySelector(".login-form"),
      header = document.querySelector(".header");
    loginForm.style.display = "none";
    loginLoader.style.display = "block";
    header.style.display = "none";
  }

  OnPremiseLogin.prototype.hideLoader = function(e) {
    var loginLoader = document.querySelector(".login-loader"),
      loginForm = document.querySelector(".login-form"),
      header = document.querySelector(".header");
      header.style.display = "block";
    loginForm.style.display = "block";
    loginLoader.style.display = "none";
  }
  