var PublishEditor = function(app) {
  var that = this;
  that.app = app;
}

PublishEditor.prototype.init = function() {
  var that = this;

  that.initViewState();
  that.initDeviceDropdown();
  that.initSearch();
  that.initPublish();
  that.initDefaults();

  that.loadWorkspaces();
  that.loadShares();
}

PublishEditor.prototype.initViewState = function() {
  var that = this;

  document.getElementById("newPrototypeBtn").addEventListener("click", (e) => {
    that.tabChanged("new");
    that.populateNewPrototypeForm(null);
  }, false);

  document.getElementById("updatePrototypeBtn").addEventListener("click", (e) => {
    that.tabChanged("update");
  }, false);

  var showMore = document.getElementById("showMore");
  showMore.addEventListener("click", (e) => {
    showMore.style.display = "none";
    document.getElementById("moreContainer").style.display = "grid";
  }, false);
}

PublishEditor.prototype.initDeviceDropdown = function() {
  var that = this;

  var devicesData = JSON.parse(localStorage.getItem("devices"));
  var filtered = devicesData.data.devices.filter(filterDevice);

  var categoriesData = devicesToCategoriesCombo(filtered, that.app.localization);
  initializeCustomSelect("deviceCombo", categoriesData, that.devicesDropdownValueChanged);

  var device = that.findBestDevice(filtered, sketchData.bounds.width, sketchData.bounds.height);
  setSelectedValue("deviceCombo", device ? device.id : null, that.devicesDropdownValueChanged);
  if (device.supportsCustomSize) {
    const deviceWidthInput = document.getElementById("deviceWidth");
    const deviceHeightInput = document.getElementById("deviceHeight");
    deviceWidthInput.value = sketchData.bounds.width;
    deviceHeightInput.value = sketchData.bounds.height;
  }
}

PublishEditor.prototype.initSearch = function() {
  var that = this;

  document.getElementById("searchBox").addEventListener("keyup", (e) => {
    var term = e.target.value;
    var shareContainers = Array.from(document.getElementsByClassName("share-container"));
    shareContainers.forEach((sc) => {
      if (term == "") {
        sc.style.display = "grid";
      } else {
        if (sc.querySelector("h4").innerText.toLowerCase().indexOf(term.toLowerCase()) > -1) {
          sc.style.display = "grid";
        } else {
          sc.style.display = "none";
        }
      }
    });
  }, false);
}

PublishEditor.prototype.initPublish = function() {
  var that = this;

  document.getElementById("publishBtn").addEventListener("click", (e) => {
    try {
      var userInfo = JSON.parse(localStorage.getItem("userInfo"));
      var tokenBuffer = localStorage.getItem("iamToken");
      var token = token ? JSON.parse(tokenBuffer).access_token : userInfo.data.token;

      // Populate params before switching page to get access to the form fields
      var params = that.populateParams();
      that.loadPublishingPage();

      this.loadAssets(params)
        .then(loadedAssets =>
          publish(that.app.apiUrl, params, loadedAssets, token)
        )
        .then((r) => {
          var result = JSON.parse(r);
          params.project.id = result.shareId;
          that.app.publishedShareId = result.shareId;
          that.app.publishedShareTitle = params.project.name;
          delete params.assets;
          localStorage.setItem("publishParams", JSON.stringify(params));
          localStorage.setItem("lastSketchFile", JSON.stringify({
            id: sketchData.sketchFile.id,
            filePath: sketchData.sketchFile.filePath,
          }));

          that.loadCompletedPage(result);
        })
        .catch((err) => {
          that.loadFailedPage(err);
        });
    } catch (err) {
      that.loadFailedPage(err);
      return;
    }
  }, false);
}

PublishEditor.prototype.initDefaults = function() {
  var that = this,
      publishBtn = document.getElementById("publishBtn"),
      prototypeTitleInput = document.getElementById("prototypeTitleInput"),
      fileParts = sketchData.sketchFile.filePath.split("/");

  var fileName = fileParts[fileParts.length - 1].split(".")[0];
  prototypeTitleInput.value = decodeURI(fileName);

  prototypeTitleInput.addEventListener("keyup", (e) => {
    var value = e.target.value;
    if (value.length > 0 && value.length <= 256) {
      publishBtn.removeAttribute("disabled");
    } else {
      publishBtn.setAttribute("disabled", "disabled");
    }
  }, false);

  var descriptionTextarea = document.getElementById("descriptionTextarea");
  descriptionTextarea.addEventListener("keyup", (e) => {
    var value = e.target.value;
    if (value.length <= 6535) {
      publishBtn.removeAttribute("disabled");
    } else {
      publishBtn.setAttribute("disabled", "disabled");
    }
  }, false);

  var passwordInput = document.getElementById("passwordInput");
  passwordInput.addEventListener("keyup", (e) => {
    var value = e.target.value;
    if (value.length <= 26) {
      publishBtn.removeAttribute("disabled");
    } else {
      publishBtn.setAttribute("disabled", "disabled");
    }
  }, false);
}

PublishEditor.prototype.loadWorkspaces = function() {
  var that = this,
      combo = document.getElementById("workspaceCombo"),
      publishBtn = document.getElementById("publishBtn"),
      updateCombo = document.getElementById("updateWorkspaceCombo"),
      shareContainers = document.getElementsByClassName("share-container"),
      createOption = function(combo, workspace) {
        var option = document.createElement("div");
        option.innerHTML = workspace.title;
        option.className = "selectable-dropdown-items";
        option.setAttribute("data-id", workspace.workspaceId);

        var itemsContainer = combo.querySelector(".select-items");
        itemsContainer.appendChild(option);
      };

  var userWorkspaces = JSON.parse(localStorage.getItem("userWorkspaces"));
  userWorkspaces
  .filter((w) => {
    return w.archivedAt == null;
  })
  .sort((a, b) => {
    var first = a.title.toUpperCase();
    var second = b.title.toUpperCase();
    if (first < second) {
      return -1;
    }

    if (second < first) {
      return 1;
    }

    return 0;
  })
  .forEach((w) => {
    createOption(combo, w);
    createOption(updateCombo, w);
  });

  initializeCustomSelect("workspaceCombo");
  initializeCustomSelect("updateWorkspaceCombo", null, (newVal) => {
    publishBtn.setAttribute("disabled", "disabled");
    that.loadShares();
  });
}

PublishEditor.prototype.loadShares = function() {
  var that = this;

  var sharesContainer = document.getElementsByClassName("shares-container")[0],
      updateCombo = document.getElementById("updateWorkspaceCombo"),
      updateComboSelectedItem = updateCombo.querySelector(".select-selected"),
      updateComboItemsContainer = updateCombo.querySelector(".select-items"),
      userShares = JSON.parse(localStorage.getItem("userShares"));

  sharesContainer.innerHTML = "";
  userShares
    .filter((s) => {
      if (updateComboSelectedItem.getAttribute("data-id") == "personal") {
        return s.workspaceId == null;
      }

      return updateComboSelectedItem.getAttribute("data-id") == s.workspaceId;
    })
    .sort((a, b) => {
      return new Date(a.updatedAt) < new Date(b.updatedAt);
    })
    .forEach((s) => {
      var shareEl = document.createElement("div");
      shareEl.innerHTML = Mustache.render(that.app.pages.shareContainer.template, {
        shareId: s.shareId,
        selected: false,
        title: s.title,
        username: s.owner.username,
        imgUrl: s.versionId != null ? that.app.apiUrl + "/shares/" + s.shareId + "/thumbs/default.png" : "img/defaultThumb.png",
        onDate: new Date(s.updatedAt).toLocaleString(),
        by: that.app.localization.strings.By,
        on: that.app.localization.strings.On
      });

      sharesContainer.appendChild(shareEl);
  });


  that.initShareHandlers();
}

PublishEditor.prototype.findBestDevice = function(devicesData, width, height) {
  const multipliers = [1, 2, 3];
  const tolerance = 1;
  const near = (length1, length2) => Math.abs(length1 - length2) <= tolerance;
  const matchSize = size => multipliers.some(m => near(size.w * m, width) && near(size.h * m, height));

  const exact = devicesData.find((d) => !d.supportsCustomSize && matchSize(d.size));
  if (exact) {
    return exact;
  }

  if (width / height < 0.7) {
    return devicesData.find((d) => d.category === "phone" && d.supportsCustomSize);
  } else {
    return devicesData.find((d) => d.category === "desktop" && d.supportsCustomSize);
  }
}

PublishEditor.prototype.populateParams = function() {
  var that = this;

  var params = {
    "project": {
      "name": null,
      "deviceId": getSelectedValue("deviceCombo"),
      "homeId": getHomeScreenId()
    },
    "sketchMetadata": {
      "pages": [sketchData.pages]
    },
    "publish": true,
    "publishOptions": {
      "description": "",
      "comments": false,
      "highlights": false,
      "password": null
    }
  };

  function getHomeScreenId() {
    var home = sketchData.screens.filter((s) => s.isHome)[0];
    if (home != null) {
      return home.id;
    }

    return sketchData.screens.length > 0 ? sketchData.screens[0].id : null;
  }

  if (that.app.action == "new") {
    params = that.populateNewShareParams(params);
  } else if(that.app.action == "updateWithChanges") {
    params = that.populateExistingShareParams(params);
    params = that.populateNewShareParams(params);
  } else {
    params = that.populateExistingShareParams(params);
  }

  return params;
}

PublishEditor.prototype.populateNewShareParams = function(params) {
  var that = this;
  var prototypeTitleInput = document.getElementById("prototypeTitleInput");
  params.project.name = prototypeTitleInput.value;

  var workspaceCombo = document.querySelector("#workspaceCombo .select-selected");
  params.workspaceId = workspaceCombo.innerHTML.toLowerCase() == "personal" ? null : workspaceCombo.getAttribute("data-id");;

  var descriptionTextarea = document.getElementById("descriptionTextarea");
  params.publishOptions.description = descriptionTextarea.value;

  var comments = document.getElementById("comments");
  params.publishOptions.comments = comments.checked;

  var highlights = document.getElementById("highlights");
  params.publishOptions.highlights = highlights.checked;

  var passwordInput = document.getElementById("passwordInput");
  params.publishOptions.password = passwordInput.value;

  if (getSelectedValue("deviceCombo").indexOf("generic") > -1) {
    var width = parseInt(document.getElementById("deviceWidth").value);
    var height = parseInt(document.getElementById("deviceHeight").value);

    params.project.customSize = {
      "w": isNaN(width) ? 0 : width,
      "h": isNaN(height) ? 0 : height
    };
  }

  return params;
}

PublishEditor.prototype.populateExistingShareParams = function(params) {
  var that = this,
      shareContainers = Array.from(document.getElementsByClassName("share-container")),
      selectedShareId = shareContainers
        .filter((sc) => sc.getAttribute("data-selected") == "true")
        .map((sc) => sc.getAttribute("data-shareId"))[0];

  if (selectedShareId == null) {
    return null;
  }

  var userShares = JSON.parse(localStorage.getItem("userShares"));
  var selectedShare = userShares.filter((us) => us.shareId == selectedShareId)[0];

  var workspaceCombo = document.querySelector("#updateWorkspaceCombo .select-selected");
  params.workspaceId = workspaceCombo.innerHTML.toLowerCase() == "personal" ? null : workspaceCombo.getAttribute("data-id");;

  params.project.id = selectedShare.shareId;
  params.project.name = selectedShare.title;

  params.publishOptions.description = selectedShare.description;
  params.publishOptions.comments = selectedShare.options.comments == true;
  params.publishOptions.highlights = selectedShare.options.highlight == true;
  params.publishOptions.password = selectedShare.password;

  return params;
}

PublishEditor.prototype.populateNewPrototypeForm = function(share) {
  var fileParts = sketchData.sketchFile.filePath.split("/");
  var fileName = fileParts[fileParts.length - 1].split(".")[0];
  var selectedShare = null;
  if (share == null) {
      selectedShare = {
        workspaceId: null,
        title: decodeURI(fileName),
        description: "",
        password: "",
        options: {
          highlight: false,
          comments: false
        }
      };
  } else {
    selectedShare = share;
  }

  if (selectedShare.viewport != null) {
    var devicesData = JSON.parse(localStorage.getItem("devices"));
    var filtered = devicesData.data.devices.filter(filterDevice);
    const device = this.findBestDevice(filtered, selectedShare.viewport.w, selectedShare.viewport.h);
    setSelectedValue("deviceCombo", device ? device.id : null, this.devicesDropdownValueChanged);

    if (device.supportsCustomSize) {
      document.getElementById("deviceWidth").value = selectedShare.viewport.w;
      document.getElementById("deviceHeight").value = selectedShare.viewport.h;
    }
  }

  var workspaceCombo = document.querySelector("#workspaceCombo .select-selected");
  if (selectedShare.workspaceId == null) {
    workspaceCombo.innerHTML = "Personal";
    workspaceCombo.setAttribute("data-id", "personal");
  } else {
    workspaceCombo.innerHTML = selectedShare.title;
    workspaceCombo.setAttribute("data-id", selectedShare.workspaceId);
  }

  if (share == null) {
    workspaceCombo.removeAttribute("disabled");
  } else {
    workspaceCombo.setAttribute("disabled", "disabled");
  }

  var prototypeTitleInput = document.getElementById("prototypeTitleInput");
  prototypeTitleInput.value = selectedShare.title != null ? selectedShare.title : "";

  var descriptionTextarea = document.getElementById("descriptionTextarea");
  descriptionTextarea.value = selectedShare.description != null ? selectedShare.description : "";

  var passwordInput = document.getElementById("passwordInput");
  passwordInput.value = selectedShare.password != null ? selectedShare.password : "";

  var highlights = document.getElementById("highlights");
  highlights.checked = selectedShare.options != null ? selectedShare.options.highlight : true;

  var comments = document.getElementById("comments");
  comments.checked = selectedShare.options.comments != null ? selectedShare.options.comments : true;
}

PublishEditor.prototype.tabChanged = function(tabName) {
  var that = this;

  that.app.action = tabName;

  var updateContainer = document.getElementsByClassName("update-prototype-container")[0];
  var newContainer = document.getElementsByClassName("new-prototype-container")[0];
  var newPrototypeBtn = document.getElementById("newPrototypeBtn");
  var updatePrototypeBtn = document.getElementById("updatePrototypeBtn");
  var publishBtn = document.getElementById("publishBtn");

  switch (tabName) {
    case "new":
      newContainer.style.display = "block";
      updateContainer.style.display = "none";
      newPrototypeBtn.classList.add("current");
      updatePrototypeBtn.classList.remove("current");

      publishBtn.removeAttribute("disabled");
      break;
    case "update":
      updateContainer.style.display = "block";
      newContainer.style.display = "none";
      updatePrototypeBtn.classList.add("current");
      newPrototypeBtn.classList.remove("current");

      var shareContainers = Array.from(document.getElementsByClassName("share-container"));
      var selectedShare = shareContainers.filter((sc) => sc.getAttribute("data-selected") == "true")[0];
      if (selectedShare == null) {
        publishBtn.setAttribute("disabled", "disabled");
      } else {
        publishBtn.removeAttribute("disabled");
      }

      break;
    default:
      break;
  }
}

PublishEditor.prototype.devicesDropdownValueChanged = function(result) {
  var deviceDimensions = document.getElementById("deviceDimensions");
  var dimenstionsContainer = document.getElementsByClassName("prototype-dimensions-container")[0];

  if (result == null || result.value == null || result.value.indexOf("generic") < 0) {
    deviceDimensions.style.display = "none";
    dimenstionsContainer.style.display = "block";
  } else {
    deviceDimensions.style.display = "grid";
    dimenstionsContainer.style.display = "grid";
  }
}

PublishEditor.prototype.initShareHandlers = function() {
  var that = this;

  var publishBtn = document.getElementById("publishBtn");
  var shareContainers = Array.from(document.getElementsByClassName("share-container"));
  shareContainers.forEach((sc) => {
    sc.addEventListener("click", (e) => {
      shareContainers.forEach((c) => {
        c.classList.remove("selectedShare");
        c.setAttribute("data-selected", "false");
      });

      e.srcElement.setAttribute("data-selected", "true");
      if (e.srcElement.tagName == "BUTTON") {
        e.srcElement.parentElement.setAttribute("data-selected", "true");
      }
      e.srcElement.classList.add("selectedShare");
      publishBtn.removeAttribute("disabled");
    });
  });

  var gearBtns = Array.from(document.getElementsByClassName("gear-btn"));
  gearBtns.forEach((gb) => {
    gb.addEventListener("click", (e) => {
      var selectedShareId = e.srcElement.getAttribute("data-shareId");
      e.srcElement.parentElement.setAttribute("data-selected", "true");

      var userShares = JSON.parse(localStorage.getItem("userShares"))
      var selectedShare = userShares.filter((us) => us.shareId == selectedShareId)[0];

      var updateContainer = document.getElementsByClassName("update-prototype-container")[0];
      updateContainer.style.display = "none";
      var newContainer = document.getElementsByClassName("new-prototype-container")[0];
      newContainer.style.display = "block";

      that.populateNewPrototypeForm(selectedShare);
      that.app.action = "updateWithChanges";
    }, false);
  });
}

PublishEditor.prototype.loadPublishingPage = function() {
  var that = this;
  that.app.navigator.navigate(that.app.mainContainer, that.app.pages.publishing);
}

PublishEditor.prototype.loadCompletedPage = function(result) {
  var that = this;
  that.app.shareId = result.shareId;
  that.app.shareLink = result.shareRunUrl;
  that.app.workspaceUrl = result.workspaceUrl;

  that.app.navigator.navigate(that.app.mainContainer, that.app.pages.completed);
}

PublishEditor.prototype.loadFailedPage = function(err) {
  var that = this;
  console.error(err);
  that.app.pages.failed.model.error = err;

  if (that.app.pages.failed.model.error == "Missing Artboard images") {
    that.app.pages.failed.model.error = that.app.localization.strings.NoArtboardMessage;
  }

  that.app.navigator.navigate(that.app.mainContainer, that.app.pages.failed);
}

PublishEditor.prototype.loadAssets = function(params) {
  const assetManifest = this.app.exportAssets();

  if (assetManifest) {
    assets = assetManifest.map(asset => ({
      sketchPageId: asset.pageId,
      sketchArtboardId: asset.artboardId,
      sketchLayerId: asset.layerId,
      name: asset.name,
      scale: asset.scale,
      format: asset.format,
    }));
  
    params.assets = assets;

    const assetPromises = assetManifest.map(asset => {
      return loadFile(asset.path).then(blob => ({ blob: blob, info: asset}));
    });
  
    return Promise.all(assetPromises);
  }

  return Promise.resolve([]);
}

var categoriesConf = {
  phone: { predefined: true, custom: true, labelKey: "CategoryPhone" },
  tablet: { predefined: true, custom: true, labelKey: "CategoryTablet" },
  desktop: { predefined: false, custom: true, labelKey: "CategoryDesktop" },
  browser: { predefined: false, custom: true, labelKey: "CategoryBrowser" },
  other: { predefined: true, custom: false, labelKey: "CategoryOther" },  // Fallback in case new categories are introduced
};

function filterDevice(device) {
  const conf = categoriesConf[device.category] || categoriesConf.other;
  if (device.supportsCustomSize && conf.custom) {
    return true;
  } else if (!device.supportsCustomSize && conf.predefined) {
    return true;
  } else {
    return false;
  }
}

function devicesToCategoriesCombo(devices, localization) {
  var categories = [];
  devices.forEach((d) => {
    const category = (d.category in categoriesConf) ? d.category : "other";

    const labelKey = categoriesConf[category].labelKey;
    if (categories.filter((c) => c.name == category).length == 0) {
      categories.push({
        name: category,
        displayName: localization.strings[labelKey],
        items: []
      });
    }

    categories
      .filter((c) => c.name == category)
      .forEach((c) => {
        c.items.push({
          id: d.id,
          displayName:  d.supportsCustomSize ? localization.strings.CustomDevice : d.name,
        });
      });
  });

  return categories;
}
